@@ -11,7 +11,7 @@ Full documentation is available at http://livestreamer.tanuki.se/.
 
 
 __title__ = "livestreamer"
-__version__ = "1.7.5"
+__version__ = "1.8.0"
 __license__ = "Simplified BSD"
 __author__ = "Christopher Rosell"
 __copyright__ = "Copyright 2011-2014 Christopher Rosell"
@@ -32,7 +32,9 @@ __credits__ = ["Christopher Rosell",
                "Andy Mikhailenko",
                "unintended",
                "Moritz Blanke",
-               "Jon Bergli Heier"]
+               "Jon Bergli Heier",
+               "Stefan Breunig",
+               "papplampe"]
 
 
 from .exceptions import (LivestreamerError, PluginError, NoStreamsError,
