@@ -3,8 +3,9 @@ import re
 from livestreamer.compat import urlparse
 from livestreamer.exceptions import PluginError
 from livestreamer.plugin import Plugin
-from livestreamer.stream import RTMPStream
-from livestreamer.utils import urlget, verifyjson, res_json
+from livestreamer.plugin.api import http
+from livestreamer.stream import HDSStream, RTMPStream
+from livestreamer.utils import verifyjson
 
 
 METADATA_URL = "https://api.dailymotion.com/video/{0}"
@@ -14,7 +15,8 @@ QUALITY_MAP = {
     "hq": "480p",
     "hd720": "720p",
     "hd1080": "1080p",
-    "custom": "live"
+    "custom": "live",
+    "auto": "hds"
 }
 RTMP_SPLIT_REGEX = r"(?P<host>rtmp://[^/]+)/(?P<app>[^/]+)/(?P<playpath>.+)"
 STREAM_INFO_URL = "http://www.dailymotion.com/sequence/full/{0}"
@@ -31,8 +33,8 @@ class DailyMotion(Plugin):
 
     def _check_channel_live(self, channelname):
         url = METADATA_URL.format(channelname)
-        res = urlget(url, params=dict(fields="mode"))
-        json = res_json(res)
+        res = http.get(url, params=dict(fields="mode"))
+        json = http.json(res)
 
         if not isinstance(json, dict):
             raise PluginError("Invalid JSON response")
@@ -47,7 +49,7 @@ class DailyMotion(Plugin):
             rpart = urlparse(url).path.rstrip("/").rpartition("/")[-1].lower()
             name = re.sub("_.*", "", rpart)
         elif ("video.gamecreds.com" in url):
-            res = urlget(url)
+            res = http.get(url)
             # The HTML is broken (unclosed meta tags) and minidom fails to parse.
             # Since we are not manipulating the DOM, we get away with a simple grep instead of fixing it.
             match = re.search("<meta property=\"og:video\" content=\"http://www.dailymotion.com/swf/video/([a-z0-9]{6})", res.text)
@@ -66,8 +68,8 @@ class DailyMotion(Plugin):
 
     def _get_rtmp_streams(self, channelname):
         self.logger.debug("Fetching stream info")
-        res = urlget(STREAM_INFO_URL.format(channelname))
-        json = res_json(res)
+        res = http.get(STREAM_INFO_URL.format(channelname))
+        json = http.json(res)
 
         if not isinstance(json, dict):
             raise PluginError("Invalid JSON response")
@@ -105,26 +107,31 @@ class DailyMotion(Plugin):
                     continue
 
                 try:
-                    res = urlget(url, exception=IOError)
+                    res = http.get(url, exception=IOError)
                 except IOError:
                     continue
 
-                match = re.match(RTMP_SPLIT_REGEX, res.text)
-                if not match:
-                    self.logger.warning("Failed to split RTMP URL: {0}",
-                                        res.text)
-                    continue
+                if quality == "hds":
+                    hds_streams = HDSStream.parse_manifest(self.session,
+                                                           res.url)
+                    streams.update(hds_streams)
+                else:
+                    match = re.match(RTMP_SPLIT_REGEX, res.text)
+                    if not match:
+                        self.logger.warning("Failed to split RTMP URL: {0}",
+                                            res.text)
+                        continue
 
-                stream = RTMPStream(self.session, {
-                    "rtmp": match.group("host"),
-                    "app": match.group("app"),
-                    "playpath": match.group("playpath"),
-                    "swfVfy": swfurl,
-                    "live": True
-                })
+                    stream = RTMPStream(self.session, {
+                        "rtmp": match.group("host"),
+                        "app": match.group("app"),
+                        "playpath": match.group("playpath"),
+                        "swfVfy": swfurl,
+                        "live": True
+                    })
 
-                self.logger.debug("Adding URL: {0}", res.text)
-                streams[quality] = stream
+                    self.logger.debug("Adding URL: {0}", res.text)
+                    streams[quality] = stream
 
         return streams
 
